
#include "XmlManager.hh"

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include <iostream>
#include <cassert>
#include <sstream>
#include <string>

XmlManager::XmlManager()
  : doc(0), xpathCtx(0)
{
  /* Init libxml */
  xmlInitParser();

  /* check version */
  LIBXML_TEST_VERSION;
}

XmlManager::~XmlManager() {
  CloseFile();
    
  /* Shutdown libxml */
  xmlCleanupParser();
    
  /* this is to debug memory for regression tests */
  xmlMemoryDump();
}

int XmlManager::OpenFile(const char* filename) {
  assert(filename);

  CloseFile();

  /* Load XML document */
  doc = xmlParseFile(filename);
  if (doc == 0) {
    std::cerr << "Error: unable to parse file \"" << filename
	      << "\"\n" << std::endl;
    return(0);
  }

  /* Create xpath evaluation context */
  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == 0) {
    std::cerr << "Error: unable to create new XPath context\n"
	      << std::endl;
    xmlFreeDoc(doc); 
    return(0);
  }

  return(1);
}

int XmlManager::CloseFile() {
  /* Cleanup of XPath data */
  xmlXPathFreeContext(xpathCtx); 
  xpathCtx = 0;

  xmlFreeDoc(doc); 
  doc = 0;

  return(1);
}

template <class ValueType>
int XmlManager::GetValue(const char* xpathExpr, ValueType* value) {
  assert(xpathExpr);
  assert(value);

  /* Evaluate xpath expression */
  xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression(xmlCharStrdup(xpathExpr),
						      xpathCtx);
  if(xpathObj == 0) {
    std::cerr << "Error: unable to evaluate XPath expression \""
	      << xpathExpr << "\"" << std::endl;
    xmlXPathFreeContext(xpathCtx); 
    xmlFreeDoc(doc); 
    return(0);
  }

  /* get value by selected nodes */
  xmlNodeSetPtr nodes = xpathObj->nodesetval;
  if(!nodes || nodes->nodeNr != 1){
    std::cerr << "Error: XPath \"" << xpathExpr
	      << "\" dosn't exist or has ambiguity" << std::endl;
    return(0);
  }
    
  std::istringstream iss((const char*)xmlNodeGetContent(nodes->nodeTab[0]));
  iss >> *value;
    
  /* Cleanup of XPath data */
  xmlXPathFreeObject(xpathObj);

  return(1);
}

//explicit instantiation
template int XmlManager::GetValue<double>(const char*, double*);
template int XmlManager::GetValue<int>(const char*, int*);
template int XmlManager::GetValue<std::string>(const char*, std::string*);

